import config
import gleam/dynamic/decode
import gleam/erlang/process
import gleam/json
import session
import storail

pub type ClientMessage {
  BuzzIn(time: Float)
  Register(game_code: String, username: String)
}

fn client_message_decoder() -> decode.Decoder(ClientMessage) {
  use variant <- decode.field("type", decode.string)
  case variant {
    "BuzzIn" -> {
      use time <- decode.field("time", decode.float)
      decode.success(BuzzIn(time:))
    }
    "Register" -> {
      use game_code <- decode.field("game_code", decode.string)
      use username <- decode.field("username", decode.string)
      decode.success(Register(game_code:, username:))
    }
    _ -> decode.failure(BuzzIn(time: 0.0), "ClientMessage")
  }
}

pub type ClientRequest {
  ClientRequest(token: String, msg: ClientMessage)
}

pub fn client_request_decoder() -> decode.Decoder(ClientRequest) {
  use token <- decode.field("token", decode.string)
  use msg <- decode.field("msg", client_message_decoder())
  decode.success(ClientRequest(token:, msg:))
}

pub type GameResponse {
  JoinResponse(username: String, token: String)
  AckBuzzer
  ResetBuzzer
  UpdatePointTotal(score: Int)
}

pub fn encode_game_response(game_response: GameResponse) -> json.Json {
  case game_response {
    JoinResponse(..) ->
      json.object([
        #("type", json.string("JoinResponse")),
        #("username", json.string(game_response.username)),
        #("token", json.string(game_response.token)),
      ])
    AckBuzzer -> json.object([#("type", json.string("AckBuzzer"))])
    ResetBuzzer -> json.object([#("type", json.string("ResetBuzzer"))])
    UpdatePointTotal(..) ->
      json.object([
        #("type", json.string("UpdatePointTotal")),
        #("score", json.int(game_response.score)),
      ])
  }
}

pub type Context {
  Context(
    config: config.Config,
    sessions: storail.Collection(session.Session),
    selector: process.Selector(ClientRequest),
  )
}
