use context::Context;
use login_screen::LoginScreen;
use macroquad::prelude::*;
use player_screen::PlayerScreen;
use screen::Screen;

mod context;
mod login_screen;
mod model;
mod player_screen;
mod screen;

pub enum State {
    Init,
    Login(LoginScreen),
    PlayerScreen(PlayerScreen),
}

impl State {
    pub fn transition_state(&self, transition: StateTransition) -> Self {
        match transition {
            StateTransition::Init | StateTransition::LeaveGame => {
                Self::Login(LoginScreen::default())
            }
            StateTransition::JoinAsPlayer => Self::PlayerScreen(PlayerScreen::default()),
        }
    }

    pub fn handle_frame(&mut self, ctx: &mut Context) {
        match self {
            Self::Init => {}
            Self::Login(login_screen) => login_screen.handle_frame(ctx),
            Self::PlayerScreen(player_screen) => player_screen.handle_frame(ctx),
        }
    }

    pub fn handle_messages(&mut self, ctx: &mut Context) -> Option<StateTransition> {
        match self {
            Self::Init => Some(StateTransition::Init),
            State::Login(login_screen) => login_screen.handle_messages(ctx),
            State::PlayerScreen(player_screen) => player_screen.handle_messages(ctx),
        }
    }
}

pub enum StateTransition {
    Init,
    JoinAsPlayer,
    LeaveGame,
}

#[macroquad::main("Play of the Game")]
async fn main() {
    let mut context = Context::new("ws://127.0.0.1:8080/ws");

    let mut state = State::Init;
    state.transition_state(StateTransition::Init);

    loop {
        if let Some(transition) = state.handle_messages(&mut context) {
            state = state.transition_state(transition);
        }

        state.handle_frame(&mut context);

        next_frame().await
    }
}
